/* bmcd_def.h COPYRIGHT FUJITSU LIMITED 2018 */
#ifndef __BMCD_DEF__
#define __BMCD_DEF__

/**
 * @brief	ioctl command declaration
 */
#define BMCD_IOCTL_MAGIC	0x0AE   /* BMC driver's ioctl magic number */
#define BMCDIOCREPORT		_IOC(_IOC_NONE, BMCD_IOCTL_MAGIC, 1, 0)
#define BMCDIOCOSDUMP		_IOC(_IOC_NONE, BMCD_IOCTL_MAGIC, 2, 0)
#define BMCDIOCERRLOG		_IOC(_IOC_NONE, BMCD_IOCTL_MAGIC, 3, 0)

/**
 * @brief	BMC driver return code declaration for export function
 */
typedef enum {
	E_NORMALEND 	= 0x00,	/* normal end */
	E_IOERROR	= 0x05,	/* reply impossible of command publication */
	E_FILEERROR	= 0x0D,	/* file access error reply */
	E_BUFFULL	= 0x10,	/* buffer full */
	E_COMERROR	= 0x12,	/* command & parameter error */
	E_INTERNALERROR	= 0x14,	/* inside error */
	E_HARDERROR	= 0x15,	/* hard error */
	E_BUSY		= 0x16,	/* busy */
	E_SYSERROR	= 0x1B,	/* system error */
	E_COMTIMEOUT	= 0x1C,	/* command timeout */
	E_NOTREADY	= 0xFF, /* non-preparations */
#ifndef __KERNEL__
	ERESTARTSYS	= 0x200	/* command killed */
#endif	/* __KERNEL__ */
} bmcd_return_t;

/**
 * @brief	BMC driver Status Report code declaration
 */
#define E_SYSRUNINFO		0x00	/* notice of system operation state */
#define E_SHUTDOWNINFO		0x01	/* notice of system cutting */
#define E_ABSHUTDOWNINFO	0x02	/* notice of system abnormality cutting */
#define E_PANICINFO		0x03	/* notice of system panic */

/**
 * @brief BMC driver ioctl function Definition for debug
 */
#define OSDUMP_NAME_SIZE  256	/* NULL終端のぶんを引く */
#define OSDUMP_DATA_SIZE  (1024 * 1024 * 2)
#define ERRLOG_DATA_SIZE  640

#ifdef __KERNEL__

/* OS DUMP ioctl data structure for debug */
struct bmcd_osdump_data {
	unsigned char	data_flag;		/* no data indicate (0:data, 1:NULL) */
	unsigned char	name_flag;		/* no name indicate (0:name, 1:NULL) */
	unsigned char	wait_flag;		/* use callback (0:no use, 1:use)    */
	unsigned char	reserve[5];		/* reserve                           */
	unsigned int	datasize;		/* dump data size                    */
	unsigned int	namesize;		/* file name size                    */
	unsigned char	name[OSDUMP_NAME_SIZE];	/* file name area                    */
	char		data[OSDUMP_DATA_SIZE];	/* dump data area                    */
} __attribute__((__packed__));

/* ERROR LOG ioctl data structure for debug */
struct bmcd_errlog_data {
	unsigned char	data_flag;		/* no data indicate (0:data, 1:NULL) */
	unsigned char	wait_flag;		/* use callback (0:no use, 1:use)    */
	unsigned char	reserve[2];		/* reserve                           */
	unsigned int	datasize;		/* error log data size               */
	char		data[ERRLOG_DATA_SIZE];	/* error log data area               */
} __attribute__((__packed__));

/**
 * @brief	BMC driver module structure declaration
 */
struct bmcd_dump_info {
	uint32_t	datasize;	/* data size Storage area */
	uint32_t	namesize;	/* file name size storage area */
	uint8_t		*filename;	/* file name storage domain pointer storage area */
	void		*info;		/* data storage domain pointer storage area */
};

struct bmcd_accident_info {
	uint32_t	size;		/* data size storage area */
	uint32_t	reserve;	/* reserved area */
	void		*info;		/* data storage domain pointer storage area */
};

/**
 * @brief	BMC driver module export prototype declaration
 */
short bmcd_osdump_req(struct bmcd_dump_info *, void(*callback_function)(short));
short bmcd_setreport(short, void(*callback_function)(short));
short bmcd_senderrlog(struct bmcd_accident_info *, void (*callback_function)(short));

#endif /* __KERNEL__ */

#endif /* __BMCD_DEF__ */
