/* bmcd_ipmi.h COPYRIGHT FUJITSU LIMITED 2018 */
#ifndef __BMCD_IPMI__
#define __BMCD_IPMI__

#include <linux/module.h>
#include "bmcd_queue.h"
#include "bmcd_def.h"

#define TX_BUFF_MAX_SIZE	65536	/* tx buffer maximum size 64Kb */

/**
 * 各IPMIコマンド用のテーブル
 */
/* System Status Report (NetFn:38h CMD:01h) */
struct bmcd_ipmi_param_report {
	unsigned char node_state;
	unsigned char add_info;
	unsigned char reserve[2];
};

/* Set Node OS Error Log (NetFn:38h CMD:31h) */
struct bmcd_ipmi_param_errlog {
	uint32_t sram_size;
	uint32_t sram_check_sum;
};

struct bmcd_ipmi_sram_errlog {
	char log[ERRLOG_DATA_SIZE];
};

/* Dump Upload Ready Notice (NetFn:38h CMD:40h) */
struct bmcd_ipmi_param_dump1 {
	uint32_t sram_size;
	uint32_t sram_check_sum;
};

struct bmcd_ipmi_sram_dump1 {
	uint32_t file_size;
	char file_name[OSDUMP_NAME_SIZE];
};

/* Dump Upload Start (NetFn:38h CMD:41h) */
struct bmcd_ipmi_sram_dump2 {
	char upload_data[TX_BUFF_MAX_SIZE];
};

/* IPMI通知イベントの種類 */
typedef enum ipmi_event {
	IE_NORMAL = 0,
	IE_RESTARTSYS,
	IE_HARDERROR
} ipmi_event_t; 

/* IPMIコマンドとの非同期やり取りを同期的に処理するためのデータ */
struct ipmi_sync_param {
	wait_queue_head_t resp_wait;  /* IPMIコマンドの応答待ち用waitキュー */
	bmcd_return_t status_code;  /* IPMIコマンドの応答ステータス解析結果
                (上位I/Fに返す戻り値) */
	ipmi_event_t	event_type;	/* イベントの種類 */
	bool end;     /* false：応答待ち、true：応答待ち解除 */
};

extern int bmcd_ipmi_init(void);
extern void bmcd_ipmi_exit(void);
extern bmcd_return_t bmcd_ipmi_send(bmcd_cmd_t, void *, void *);
extern void bmcd_ipmi_panic_set(int);
extern int bmcd_ipmi_panic_get(void);

#endif /* __BMCD_IPMI__ */
