/* bmcd_log.h COPYRIGHT FUJITSU LIMITED 2018 */
#ifndef __BMCD_LOG__
#define __BMCD_LOG__

/* 開発者向け汎用ログ用デバッグレベル */
typedef enum {
	BMCD_ERROR = 0,
	BMCD_WARN,
	BMCD_INFO,
	BMCD_DEBUG,
	BMCD_LOG_LEVEL_NUM
} bmcd_log_common_t;

/* IPMIメッセージログ用デバッグレベル */
typedef enum {
	BMCD_REQ = 0,
	BMCD_RESP,
	BMCD_LOG_IO_NUM
} bmcd_log_ipmi_t;

/* bmcd_log_openのエラーコード */
#define BMCD_LOG_OPEN_FAILED_IPMIMSG	-1
#define BMCD_LOG_OPEN_FAILED_COMMON	-2
#define BMCD_LOG_CREATE_WORKQUEUE	-3

/**
 * ログレベル
 * 	開発者向け汎用ログ用デバッグレベルの出力をビットで制御する
 */
typedef enum {
	BMCD_LOG_LEVEL_1 = 0x01,	/* ERROR が有効 */
	BMCD_LOG_LEVEL_2 = 0x03,	/* ERROR/WARN が有効 */
	BMCD_LOG_LEVEL_3 = 0x07,	/* ERROR/WARN/INFO が有効 */
	BMCD_LOG_LEVEL_4 = 0x0F,	/* ERROR/WARN/INFO/DEBUG が有効 */
} bmcd_log_level_t;

/* ログ用ワークキュー情報定義 */
typedef struct log_work_struct {
	struct work_struct ws;
	struct file *fp;
	char *msg;
} log_work_struct_t;

extern void bmcd_log_init(void);
extern int bmcd_log_open(void);
extern void bmcd_log_close(void);
extern void bmcd_log_ipmimsg(bmcd_log_ipmi_t, unsigned char, unsigned char, const char *, unsigned int);
extern void bmcd_log_print(bmcd_log_common_t, const char *, int, const char *, ...);

extern void bmcd_log_level_set(bmcd_log_level_t);
extern bmcd_log_level_t bmcd_log_level_get(void);

#define BMCD_LOG(LEVEL, MSG, ...)	\
	bmcd_log_print(LEVEL, __FILE__, __LINE__, MSG, ## __VA_ARGS__);

#endif /* __BMCD_LOG__ */
