/**
 *  * Copyright(c) 2016 FUJITSU LIMITED All rights reserved.
 *   */

/*************************************************************
 * include
 *************************************************************/
#include <linux/kernel.h>
#include <linux/string.h>
#include "bmcd_common.h"

/*************************************************************
 * define
 *************************************************************/
#define BMCD_IPMI_CMDSTR_SIZE	6

/*************************************************************
 * function
 *************************************************************/
/**
 * @brief bmcd_sysmsg_to_ipmistr
 * @param bmcd_cmd_t(コマンドコード)
 * @note 引数のコマンドコードから、IPMIコマンドコードの文字列を取得する
 */
char *bmcd_sysmsg_to_ipmistr(int command)
{
	static char name[BMCD_CMD_IPMI_NUM][BMCD_IPMI_CMDSTR_SIZE] = {
		"38:01", "38:02", "38:31", "38:40", "38:41", "38:42"};

	if ((command < 0) || (command >= BMCD_CMD_IPMI_NUM))
		return "unknown code";
	return name[command];
}

/**
 * @brief bmcd_sysmsg_result
 * @param BMCドライバの外部向け戻り値
 * @param bmcd_cmd_t(コマンドコード)
 * @note IPMIコマンドの実行結果に伴うエラーをsyslogへ出力する
 */
void bmcd_sysmsg_result(short result, int command)
{
	char name[BMCD_IPMI_CMDSTR_SIZE];

	memcpy(name, bmcd_sysmsg_to_ipmistr(command), BMCD_IPMI_CMDSTR_SIZE);

	switch (result) {
	case E_IOERROR:		/* reply impossible of command publication */
		BMCD_SYSMSG_ERRR(23, "The I/O error occurred. [%s]\n", name);
		break;
	case E_BUFFULL:		/* buffer full */
		BMCD_SYSMSG_ERRR(7, "BMC could not allocate enough memory requested. [%s]\n", name);
		break;
	case E_HARDERROR:	/* hard error */
		BMCD_SYSMSG_ERRR(6, "Detected hard error. [%s]\n", name);
		break;
	case E_BUSY:		/* busy */
		BMCD_SYSMSG_INFO(1011, "Command Busy. Another command is executing it. [%s]\n", name);
		break;
	case E_SYSERROR:	/* system error */
		BMCD_SYSMSG_ERRR(22, "An unexpected error occurred. [%s]\n", name);
		break;
	case E_COMTIMEOUT:	/* command timeout */
		BMCD_SYSMSG_ERRR(10, "Command time out. [%s]\n", name);
		break;
	case E_NOTREADY:	/* non-preparations */
		BMCD_SYSMSG_ERRR(24, "BMC is not ready to accept the command. [%s]\n", name);
		break;
	case E_COMERROR:	/* command & parameter error */
		BMCD_SYSMSG_ERRR(1, "Invalid parameter and command. [%s]\n", name);
		break;
	case E_INTERNALERROR:	/* inside error */
		BMCD_SYSMSG_ERRR(5, "Detected an internal error of the IPMI command.. [%s]\n", name);
		break;
	case E_NORMALEND:	/* normal end */
		BMCD_LOG(BMCD_INFO, "Command normal end. [%s]\n", name);
		break;
	default:
		BMCD_LOG(BMCD_WARN, "Unknown code[%d]. [%s]\n", result, name);
		break;
	}
}
