/* Copyright(c) 2016 FUJITSU LIMITED All rights reserved. 
 * Kernel level system log facility 
 */

#if !defined(XOS_SFT_COM_COM_SYSLOG_H)
#define XOS_SFT_COM_COM_SYSLOG_H

#if defined(__KERNEL__)

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/compiler.h>

/**
 *  製品名
 */
#define XOS_COM_LOG_PRODUCT_XOS "xos"

/**
 *  メッセージ種別  
 */
#define XOS_COM_LOG_TYPE_DEBUG  "[DEBG]"
#define XOS_COM_LOG_TYPE_INFO   "[INFO]"

/* 
 * 以下は運用ソフトメッセージ規約に
 * 定義されているメッセージ種別
 */
#define XOS_COM_LOG_TYPE_NOTICE "[NOTE]"
#define XOS_COM_LOG_TYPE_WARN   "[WARN]"
#define XOS_COM_LOG_TYPE_ERROR  "[ERR.]"
#define XOS_COM_LOG_TYPE_EMERG  "[EMRG]"

/*
 * 改行制御  
 */
static inline void xos_com_cleanup_trail(const char *fmt) {

	size_t len = 0;

	if (fmt == NULL)
		goto error_out;

	len = strlen(fmt);
	if ( (len == 0) || (fmt[len - 1] != '\n') )
		printk("\n");

error_out:
	return;
}

/**
 * return base file name of argument string(trim directory name).
 * 
 * @param filename_p pointer to file name.
 */
static inline char * xos_com_basename(char * filename_p) {
	char * p = NULL;
	char * ret_p = filename_p;
	
	for ( p = filename_p ; *p != '\0' ; p++ ) {
		if ( *p == '/' ) {
			ret_p = p + 1;
		}
	}
	return ret_p;
}

/* message example
 * [type] xos comp msgid  -  file:XXX.c function:YYY line:ZZZ args */

#define xos_com_print(level, type, comp, msgid, jobid, fmt, arg...) 		\
do {							     			\
	printk(level type " " XOS_COM_LOG_PRODUCT_XOS " " comp " "		\
		"%04d %s file:%s function:%s line:%d " fmt,			\
		msgid, jobid ? jobid : "-", xos_com_basename(__FILE__),		\
		__FUNCTION__, __LINE__,##arg);					\
	xos_com_cleanup_trail(fmt);						\
} while (0)

#define xos_com_plain_print(fmt, arg...)	printk(fmt, ##arg)

/** デバッグメッセージをカーネルログに記録する.
    @param[in] comp   コンポーネント名(文字列)
    @param[in] msgid  メッセージID(数値)
    @param[in] jobid  ジョブID(文字列) *** デフォルトは"-"を表示する ***
    @param[in] fmt 書式指定文字列
    @param[in] arg 可変個引数
    割り込み禁止の要否:否
    呼び出しコンテキスト: 不問
    前置条件: なし(最も低レベルに位置するデバッグ支援マクロなので)
    後置条件: なし(最も低レベルに位置するデバッグ支援マクロなので)
   ツールによる関数/マクロシーケンス出力時に, 除外できるように例外的に,
   xos_以外の名称でマクロ名を開始している.
   CONFIG_XOS_DEBUG_SYSLOG有効時にのみ出力される.
 */
#if defined(CONFIG_XOS_DEBUG_SYSLOG)
#define xos_com_debug_msg(comp, msgid, jobid, fmt, arg...)			\
		xos_com_print(KERN_DEBUG, XOS_COM_LOG_TYPE_DEBUG, comp, msgid, jobid, fmt, ##arg)
#else  /*  !CONFIG_XOS_DEBUG_SYSLOG  */
#define xos_com_debug_msg(comp, msgid, jobid, fmt,arg...) do{}while(0)
#endif   /*  CONFIG_XOS_DEBUG_SYSLOG  */


/** 行番号情報なしで出力する
    @param[in] fmt  書式指定文字列
    @param[in] arg  可変個引数
    割り込み禁止の要否:否
    呼び出しコンテキスト: 不問
    前置条件: なし
    後置条件: なし
 */
#define xos_com_plain_msg(fmt, arg...)						\
	xos_com_plain_print(fmt, ##arg)						\

/** INFORMATION メッセージをカーネルログに記録する.
    @param[in] fmt 書式指定文字列
    @param[in] arg 可変個引数
    割り込み禁止の要否:否
    呼び出しコンテキスト: 不問
    前置条件: なし
    後置条件: なし
 */
#define xos_com_info_msg(comp, msgid, jobid, fmt, arg...) 			\
		xos_com_print(KERN_INFO, XOS_COM_LOG_TYPE_INFO, comp, msgid, jobid, fmt, ##arg)

/** ERROR メッセージをカーネルログに記録する.
 @param[in] comp   コンポーネント名(文字列)
 @param[in] msgid  メッセージID(数値)
 @param[in] jobid  ジョブID(文字列) *** デフォルトは"-"を表示する ***
 @param[in]  fmt  書式指定文字列
 @param[in]  arg  可変個引数
 割り込み禁止の要否:否
 呼び出しコンテキスト: 不問
 前置条件: なし
 後置条件: なし
 */
#if !defined(XOS_COM_NO_ERR_MSG)
#define xos_com_err_msg(comp, msgid, jobid, fmt, arg...) 			\
		xos_com_print(KERN_ERR, XOS_COM_LOG_TYPE_ERROR, comp, msgid, jobid, fmt, ##arg)
#else
#define xos_com_err_msg(comp, fmt,arg...) do{}while(0)
#endif  /*  !XOS_COM_NO_ERR_MSG  */

/** NOTICE メッセージをカーネルログに記録する.
 @param[in] comp   コンポーネント名(文字列)
 @param[in] msgid  メッセージID(数値)
 @param[in] jobid  ジョブID(文字列) *** デフォルトは"-"を表示する ***
 @param[in]  fmt  書式指定文字列
 @param[in]  arg  可変個引数
 割り込み禁止の要否:否
 呼び出しコンテキスト: 不問
 前置条件: なし
 後置条件: なし
 */
#define xos_com_notice_msg(comp, msgid, jobid, fmt, arg...)			\
		xos_com_print(KERN_NOTICE, XOS_COM_LOG_TYPE_NOTICE, comp, msgid, jobid, fmt, ##arg)

/** WARNING メッセージをカーネルログに記録する.
 @param[in] comp   コンポーネント名(文字列)
 @param[in] msgid  メッセージID(数値)
 @param[in] jobid  ジョブID(文字列) *** デフォルトは"-"を表示する ***
 @param[in]  fmt  書式指定文字列
 @param[in]  arg  可変個引数
 割り込み禁止の要否:否
 呼び出しコンテキスト: 不問
 前置条件: なし
 後置条件: なし
 */
#define xos_com_warn_msg(comp, msgid, jobid, fmt, arg...)			\
		xos_com_print(KERN_WARNING, XOS_COM_LOG_TYPE_WARN, comp, msgid, jobid, fmt, ##arg)


/** EMERGENCY メッセージをカーネルログに記録する.
 @param[in] comp   コンポーネント名(文字列)
 @param[in] msgid  メッセージID(数値)
 @param[in] jobid  ジョブID(文字列) *** デフォルトは"-"を表示する ***
 @param[in]  fmt  書式指定文字列
 @param[in]  arg  可変個引数
 割り込み禁止の要否:否
 呼び出しコンテキスト: 不問
 前置条件: なし
 後置条件: なし
 */
#define xos_com_emerg_msg(comp, msgid, jobid, fmt, arg...)			\
		xos_com_print(KERN_EMERG, XOS_COM_LOG_TYPE_EMERG, comp, msgid, jobid, fmt, ##arg)

#endif  /*  __KERNEL__  */

#endif  /*  XOS_SFT_COM_COM_SYSLOG_H  */
